#ifndef __TOAST_LOGGER_H__
#define __TOAST_LOGGER_H__

#ifdef TOASTLOGGER_EXPORTS
#define TOASTLOGGER_API     __declspec(dllexport)
#else
#define TOASTLOGGER_API     __declspec(dllimport)
#endif

#include "ToastLoggerConfiguration.h"
#include "ToastLoggerUserFields.h"

typedef enum _TOAST_LOGGER_LEVEL
{
    LOG_LEVEL_DEBUG = 0,
    LOG_LEVEL_INFO = 1,
    LOG_LEVEL_WARN = 2,
    LOG_LEVEL_ERROR = 3,
    LOG_LEVEL_FATAL = 4,
    LOG_LEVEL_NONE = 5
} TOAST_LOGGER_LEVEL;

namespace toast
{
    namespace logger
    {
        class TOASTLOGGER_API ToastLogger
        {
        public:
            virtual void initialize(ToastLoggerConfiguration* loggerConfiguration) = 0;

            virtual const char* getUserId() = 0;
            virtual void setUserId(const char* userId) = 0;

            virtual void addUserField(const char* key, const char* value) = 0;
            virtual void removeUserField(const char* key) = 0;
            virtual void clearUserFileds() = 0;

            virtual void log(TOAST_LOGGER_LEVEL logLevel, const char* message, ToastLoggerUserFields* userFields = nullptr) = 0;
            virtual void log(TOAST_LOGGER_LEVEL logLevel, const wchar_t* message, ToastLoggerUserFields* userFields = nullptr) = 0;

            void debug(const wchar_t* message, ToastLoggerUserFields* userFields = nullptr)
            {
                log(LOG_LEVEL_DEBUG, message, userFields);
            }

            void info(const wchar_t* message, ToastLoggerUserFields* userFields = nullptr)
            {
                log(LOG_LEVEL_INFO, message, userFields);
            }

            void warn(const wchar_t* message, ToastLoggerUserFields* userFields = nullptr)
            {
                log(LOG_LEVEL_WARN, message, userFields);
            }

            void error(const wchar_t* message, ToastLoggerUserFields* userFields = nullptr)
            {
                log(LOG_LEVEL_ERROR, message, userFields);
            }

            void fatal(const wchar_t* message, ToastLoggerUserFields* userFields = nullptr)
            {
                log(LOG_LEVEL_FATAL, message, userFields);
            }

            void debug(const char* message, ToastLoggerUserFields* userFields = nullptr)
            {
                log(LOG_LEVEL_DEBUG, message, userFields);
            }

            void info(const char* message, ToastLoggerUserFields* userFields = nullptr)
            {
                log(LOG_LEVEL_INFO, message, userFields);
            }

            void warn(const char* message, ToastLoggerUserFields* userFields = nullptr)
            {
                log(LOG_LEVEL_WARN, message, userFields);
            }

            void error(const char* message, ToastLoggerUserFields* userFields = nullptr)
            {
                log(LOG_LEVEL_ERROR, message, userFields);
            }

            void fatal(const char* message, ToastLoggerUserFields* userFields = nullptr)
            {
                log(LOG_LEVEL_FATAL, message, userFields);
            }
        };
    }
}

extern "C" TOASTLOGGER_API toast::logger::ToastLogger* GetToastLogger();
extern "C" TOASTLOGGER_API void DestroyToastLogger();

#endif  // __TOAST_LOGGER_H__